/*---------------------------------------------------------------------------
 * Copyright (C) 2005 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */
/******************************************************************************
 * This demo for the MAXQ2000 plays audio samples that have passed through 
 * a simple digital filter, implemented with the Hardware Multiply-Accumulate
 * unit of the MAXQ2000.  A 3-second audio message plus numerous filters are
 * stored in code space.  The LCD screen displays which filter the audio
 * will pass through if the 'play' button (SW5) is selected.  Different 
 * filters can be selected with the SW4 button.  
 *
 * Features demostrated in this demo:
 *   1) LCD controller
 *   2) Timers
 *   3) External Interrupts
 *   4) SPI communication
 *   5) Analog to digital conversion with external hardware (MAX1407)
 *   6) Multiply-Accumulate
 *
 * Functional overview:
 *   When the play button is pressed, the timer interrupt is enabled.  The
 *   timer is configured to run at 8 kHz, which is the sampling rate of the
 *   original audio.  On each timer interrupt, we figure out which filter 
 *   we are using and execute that filter on the audio data.  Only FIR 
 *   filters (zeroes) are supported in this version of the software--it would 
 *   not be too difficult to add IIR support if we dedicated RAM storage for 
 *   a number of samples equal to the number of pole filter taps.
 *
 *   After 24000 samples are played, the timer disables itself, waiting for 
 *   the play button to be pressed again.
 *
 * Hardware Requirements:
 *   Please see the application note "Audio Filtering with the MAXQ2000" for
 *   information on the required hardware.  This application uses the MAX1407
 *   present on the Evaluation Kit to output the audio signal.  It can be 
 *   easily connected to computer speakers to hear the output.
 *    
 *****************************************************************************/
#include <intrinsics.h>
#include <iomaxq200x.h>
#include "maxq2000_gpio.h"
#include "maxq2000ev_1407.h"
#include "maxq2000ev_lcd.h"

/*
 * Assembler function that simply gets the audio sample at the specified
 * offset.  No filtering is performed: this function is only used if 'ALL'
 * is selected as the filter (Allpass)
 */
extern unsigned int getsample(unsigned int addr);

/*
 * Assembler function that gets the audio sample at the specified offset
 * and runs it through the specified filter.  See the file 'data.asm' for
 * information on how the filters are stored.
 */
extern unsigned int getfilteredsample(unsigned int addr, unsigned int filtertype);

/*
 * The different filters available to this application.  New filters can
 * easily be added by:
 *  1 adding another define and incrementing the value of 'INVALID_FILTER_MODE'
 *  2 adding a new display name to 'showFilter()'
 *  3 adding the filter specification to the file data.asm
 *  4 adding the filter to the lookup table 'filter_table' in data.asm
 */
#define FILTER_ALLPASS      0
#define FILTER_HIGHPASS     1
#define FILTER_HIGHPASS2    2
#define FILTER_LOWPASS      3
#define FILTER_LOWPASS2     4
#define FILTER_LOWPASS3     5
#define FILTER_BANDPASS     6
#define FILTER_BANDPASS2    7
#define INVALID_FILTER_MODE 8

/*
 * A few definitions of letters used to display the filter names.
 * We are a little limited with the 7-segment display to display
 * letters, so:
 *   ALL  == Allpass filter
 *   HI   == Highpass filter
 *   LO   == Lowpass filter
 *   bP   == Bandpass filter
 */
#define LCD_NONE            0
#define LCD_H               0x76
#define LCD_I               0x30
#define LCD_L               0x38
#define LCD_O               0x3f
#define LCD_A               0x77
#define LCD_B               0x7c
#define LCD_P               0x73
#define LCD_S               0x6d

/*
 * Current index into the audio recording that we are playing.  Reset to 0
 * when the play button is pressed.
 */
unsigned int current_address;

/*
 * The current filter we are using.  Cycle around when the 'filter select'
 * button is pressed.
 */
unsigned int filter_mode;

/******************************************************************************
 * Setup the timer to run at about 8 kHx.  This is decent quality for voice 
 * audio.  Use the system clock divided by 1 as the timer source for best
 * accuracy.
 */
void initTimer2()
{
    //
    // Set up timer number 2 to give us periodic interrupts, when we will 
    // output an audio sample.  I want 8 kHz, or 125 uS. That is about 1688
    // cycles at 13.5 MHz.  10000h - 1688d = F968h
    //
    T2V2 = 0xF968;
    T2R2 = 0xF968;
    T2C2 = 0x0002;    // no compare, don't want that interrupt firing
  
    T2CFG2 = 0x00;    // use clock divided by 1
    T2CNB2 = 0x00;
    T2CNA2 = 0x88;    // start the timer, and enable interrupts
}

/******************************************************************************
 * Setup the external interrupts for EI 10 and 15 (pins P5.2 and P7.1).
 */
void initPushButtons()
{
    gpio_setinterruptenable(10);
    gpio_setinterruptenable(15);
    gpio_setedgeselect(10, FALLING_EDGE);
    gpio_setedgeselect(15, FALLING_EDGE);
    IMR |= 0x02;                          // enable interrupts for module 1
}


/******************************************************************************
 * The interrupt handler for module 4, which is my timer 2 interrupt.
 */
#pragma vector = 4
__interrupt void timer2Interrupt()
{
    unsigned int sample;
    if (T2CNB2 & 0x08)   // then it is timer 2 overflow
    {
        // get an audio sample
        if (filter_mode==FILTER_ALLPASS)
            sample = getsample(current_address);
        else 
            sample = getfilteredsample(current_address, filter_mode);
        // normalize it for our circuit
        sample = sample ^ 0x8000;
        // write to the DAC
        max1407_writeregister(MAX1407REG_DAC2, sample, LENGTH_16);
        current_address++;
        if (current_address==24000)
        {
            // turn off interrupts for module 4 if we are at the end
            IMR &= 0xEF;      
        }

        // clear the interrupt condition
        T2CNB2 = T2CNB2 & 0xF7;
    }
    else
    {
        // we are in trouble...not sure what interrupt this is!
    }
}


/******************************************************************************
 * Display the name of the current filter.  Filters are cycled through using
 * the SW4 button.
 */
void showFilter()
{
    LCD4 = LCD_NONE;
    switch (filter_mode)
    {
        case FILTER_ALLPASS   : LCD3 = LCD_A; LCD2 = LCD_L; LCD1 = LCD_L;         LCD0 = LCD_NONE; break;
        case FILTER_HIGHPASS  : LCD3 = LCD_H; LCD2 = LCD_I; LCD1 = LCD_NONE;      LCD0 = LCD_NONE; break;
        case FILTER_HIGHPASS2 : LCD3 = LCD_H; LCD2 = LCD_I; LCD1 = LCD_PATTERN_2; LCD0 = LCD_NONE; break;
        case FILTER_LOWPASS   : LCD3 = LCD_L; LCD2 = LCD_O; LCD1 = LCD_NONE;      LCD0 = LCD_NONE; break;
        case FILTER_LOWPASS2  : LCD3 = LCD_L; LCD2 = LCD_O; LCD1 = LCD_PATTERN_2; LCD0 = LCD_NONE; break;
        case FILTER_LOWPASS3  : LCD3 = LCD_L; LCD2 = LCD_O; LCD1 = LCD_PATTERN_3; LCD0 = LCD_NONE; break;
        case FILTER_BANDPASS  : LCD3 = LCD_B; LCD2 = LCD_P; LCD1 = LCD_NONE;      LCD0 = LCD_NONE; break;
        case FILTER_BANDPASS2 : LCD3 = LCD_B; LCD2 = LCD_P; LCD1 = LCD_PATTERN_2; LCD0 = LCD_NONE; break;
    }
    
}

/******************************************************************************
 * The interrupt handler for module 1, including my external interrupts.
 * Must be careful in here to make sure I react only to the interrupts I 
 * want, since some pins are dual purpose.
 */
#pragma vector = 1
__interrupt void pushButtonInterrupt()
{
  unsigned char temptime;
  // check for interrupt 10 (0x04) or interrupt 15 (0x80)
  if (EIF1 & 0x04)
  {
      // interrupt 10...select the next filter
      filter_mode++;
      if (filter_mode==INVALID_FILTER_MODE)
          filter_mode = FILTER_ALLPASS;
      showFilter();
  }
  if (EIF1 & 0x80)
  {
      // interrupt 15...start playing audio samples
      
      // set the address of the audio sample to the beginning
      current_address = 0;
      // turn on interrupts for module 4 (timer 2)
      IMR |= 0x10;      
  }
  // little software debouncing delay...wait 50/256th of a second
  temptime = RTSS;
  temptime += 50;
  while (RTSS != temptime) {}          
  // clear the interrupt condition
  EIF1 = 0;                               
}

/******************************************************************************
 * Initialize the 1407, timer, pushbuttons, LCD, and RTC.  Other than that, 
 * this application is interrupt driven, so this function goes into an infinite
 * loop and does nothing.
 */
void main()
{
  current_address = 0;
  filter_mode = FILTER_ALLPASS;
    
  max1407_init();
  max1407_writeregister(MAX1407REG_DAC1, 0x8000, LENGTH_16);

  initTimer2();
  initPushButtons();
  
  rtc_init();
  lcd_init();
  showFilter();  
  
  __enable_interrupt();
  
  while(1);
}

