/* --------------------------------------------------------------------------------------
 *  Copyright (C) 2004 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 *
 *  Module:        gpio
 *  Description:   General Purpose I/O Convenience Functions Header File
 *  Filename:      maxq2000_gpio.h
 *  Date:          October 13, 2004
 *  Version:       1.00
 *
 *  Function declarations for the GPIO convenience library that goes along with the 
 *  IAR-MAXQ2000 demonstration software.
 *
 * --------------------------------------------------------------------------------------
 */
#ifndef __MAXQ2000_GPIO__
#define __MAXQ2000_GPIO__

#define PORTPIN_0   1
#define PORTPIN_1   2
#define PORTPIN_2   4
#define PORTPIN_3   8
#define PORTPIN_4   16
#define PORTPIN_5   32
#define PORTPIN_6   64
#define PORTPIN_7   128
#define PORTPIN_ALL 255

#define RISING_EDGE  0
#define FALLING_EDGE 1

/*******************************************************************************
 * Function: gpio_setoutputpin
 *
 * Sets the bits defined by 'pinmask' as output pins in the port specified by
 * 'portnumber'.  
 * 
 * Input:    portnumber - Port number to set output pins for.  For the MAXQ2000,
 *                        should be between 0 and 7.
 *           pinmask - Bit mask of the pins that will be configured as outputs.
 *                     Use the values PORTPIN_0, PORTPIN_1, etc. to create the
 *                     bitmask.
 * Output:   None
 * 
 * 
 * Example:  To set pins 3 and 5 in port 4 as outputs...
 *                 gpio_setoutputpin(4, PORTPIN_3 | PORTPIN_5);
 *           To set all the pins on port 0 to output...
 *                 gpio_setoutputpin(0, PORTPIN_ALL);
 *
 ******************************************************************************/
void gpio_setoutputpin(int portnumber, int pinmask);

/*******************************************************************************
 * Function: gpio_setinputpin
 *
 * Sets the bits defined by 'pinmask' as input pins in the port specified by
 * 'portnumber'.  
 * 
 * Input:    portnumber - Port number to set input pins for.  For the MAXQ2000,
 *                        should be between 0 and 7.
 *           pinmask - Bit mask of the pins that will be configured as inputs.
 *                     Use the values PORTPIN_0, PORTPIN_1, etc. to create the
 *                     bitmask.
 * Output:   None
 * 
 * 
 * Example:  To set pins 3 and 5 in port 4 as inputs...
 *                 gpio_setinputpin(4, PORTPIN_3 | PORTPIN_5);
 *           To set all the pins on port 0 to input...
 *                 gpio_setinputpin(0, PORTPIN_ALL);
 *
 ******************************************************************************/
void gpio_setinputpin(int portnumber, int pinmask);

/*******************************************************************************
 * Function: gpio_setedgeselect
 *
 * Sets the external interrupt edge select for the specified external interrupt
 * as indicated by the 'risefall' parameter.
 * 
 * Input:    interrupt_number - External interrupt to set the edge select for.
 *                              On the MAXQ2000, this should be between 0 and 15
 *           risefall - Value to indicate if the edge select should be rising
 *                      or falling edge.  Use the values RISING_EDGE and 
 *                      FALLING_EDGE.
 * Output:   None
 * 
 * Example:  To set external interrupt 7 to trigger on the rising edge...
 *                 gpio_setedgeselect(7, RISING_EDGE);
 *           To set external interrupt 12 to trigger on the falling edge...
 *                 gpio_setedgeselect(12, FALLING_EDGE);
 *
 ******************************************************************************/
void gpio_setedgeselect(int interrupt_number, int risefall);

/*******************************************************************************
 * Function: gpio_setinterruptenable
 *
 * Enables the interrupt for an external interrupt.  Note that this function
 * does not enable the interrupt in the IMR mask, only in the external interrupt
 * control registers.  You may still need to enable the module interrupts with 
 * the IMR mask.
 * 
 * Input:    interrupt_number - External interrupt to enable the interrupt for.
 *                              On the MAXQ2000, this should be between 0 and 15
 * Output:   None
 * 
 * 
 * Example:  To enable interrupts for external interrupt 3...
 *                 gpio_setinterruptenable(3);
 *
 ******************************************************************************/
void gpio_setinterruptenable(int interrupt_number);

/*******************************************************************************
 * Function: gpio_clearinterruptenable
 *
 * Disables (clears) the interrupt for an external interrupt.  Note that this 
 * function does not clear anything in the IMR mask, only in the external 
 * interrupt control registers.  Other interrupts in the same module may still 
 * cause interrupts if the IMR mask allows it.
 * 
 * Input:    interrupt_number - External interrupt to disable the interrupt for.
 *                              On the MAXQ2000, this should be between 0 and 15
 * Output:   None
 * 
 * 
 * Example:  To disable interrupts for external interrupt 3...
 *                 gpio_clearinterruptenable(3);
 *
 ******************************************************************************/
void gpio_clearinterruptenable(int interrupt_number);

#endif
