/* --------------------------------------------------------------------------------------
 *  Copyright (C) 2004 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 *
 *  Module:        rtc
 *  Description:   Real-Time Clock Convenience Functions Header File
 *  Filename:      maxq2000_rtc.h
 *  Date:          October 13, 2004
 *  Version:       1.00
 *
 *  Function declarations for the RTC convenience library that goes along with the 
 *  IAR-MAXQ2000 demonstration software.
 *
 * --------------------------------------------------------------------------------------
 */
#ifndef __MAXQ2000_RTC__
#define __MAXQ2000_RTC__

#define SECONDS_PER_DAY     86400L
#define SECONDS_PER_HOUR     3600
#define SECONDS_PER_MINUTE     60

/*******************************************************************************
 * Function: rtc_init
 *
 * Initializes the real-time clock on the MAXQ2000.  Sets the real-time clock 
 * enable bit, and then disables writes to the real-time clock enable.
 * 
 * Input:    None
 * Output:   None
 *
 ******************************************************************************/
void rtc_init();

/*******************************************************************************
 * Function: rtc_setsecondsalarm
 *
 * Sets the alarm registers for the real-time clocks 'Seconds' alarm (not the
 * sub-seconds alarm).  Enables the real-time clock alarm for time-of-day.
 * 
 * Input:    t - value to set the time-of-day alarm (32 bits)
 * Output:   None
 *
 ******************************************************************************/
void rtc_setsecondsalarm(unsigned long t);

/*******************************************************************************
 * Function: rtc_getseconds
 *
 * Returns the current value of the high and low seconds registers of the 
 * MAXQ2000 real-time clock.  Note that this method does not disable the 
 * real-time clock to take its reading, so it is possible  (although highly
 * unlikely) that you could get an invalid reading on a low->high register 
 * rollover.
 * 
 * Input:    None
 * Output:   Current value of real-time clock seconds registers (32 bits)
 *
 ******************************************************************************/
unsigned long rtc_getseconds();

/*******************************************************************************
 * Function: rtc_setseconds
 *
 * Sets the current value of the real-time clock's seconds registers (i.e. sets 
 * the current time stored in the real time clock).
 * 
 * Input:    t - value to load into the real-time clock's seconds registers
 * Output:   None
 *
 * Note:     The RCNT register will be set to 8001h at the end of this function.
 *           If anything was enabled (like an alarm), it will need to be 
 *           re-enabled.
 *
 ******************************************************************************/
void rtc_setseconds(unsigned long t);

/*******************************************************************************
 * Function: rtc_translate
 *
 * Convenience function to translate a total number of seconds into a number of
 * days, hours (mod 24), minutes (mod 60), and seconds (mod 60).
 * 
 * Input:    totalseconds - Number of seconds to be translated into a more 
 *                          manageable form.  Presumably, this is the value
 *                          given by the real-time clock.
 *           seconds      - Number of seconds into the minute.
 *           minutes      - Number of minutes into the hour. 
 *           hours        - Number of hours into the day.
 *           days         - Number of whole days represented by totalseconds.
 * Output:   None.  All 'output' is done through pointers passed as inputs.
 *
 * Note:     This function does not actually use any RTC hardware.  It is 
 *           provided for convenience.
 *
 ******************************************************************************/
void rtc_translate(unsigned long totalseconds, int* seconds, int* minutes, int* hours, int* days);

#endif
