/* ----------------------------------------------------------------------------
 *  Copyright (C) 2004 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 *
 *  Module:        maxq2000_spi
 *  Description:   Utility functions for configuring and using the MAXQ2000's
 *                 SPI module.
 *  Filename:      maxq2000_spi.h
 *  Date:          November 2, 2004
 *  Version:       1.00
 *
 * ---------------------------------------------------------------------------*/

#ifndef __MAXQ2000_SPI__
#define __MAXQ2000_SPI__

/******************************************************************************
 * Function:  spi_enable
 *
 * Activates/Deactivates the Serial Peripheral Interface (SPI).  This funtion
 * sets the value of the SPIEN bit of the SPICN register based on the input
 * value.
 *
 * Input:     spien - 0 will disable the SPI module and any non-zero value will
 *                    enable it.
 *
 * Output:    None
 *
 * Example:   To enable the SPI module...
 *                  spi_enable(1);
 *
 ******************************************************************************/
void spi_enable(const unsigned char spien);

typedef enum {SLAVE_MODE, MASTER_MODE} spi_mode;

/******************************************************************************
 * Function: spi_setmode
 *
 * Places the SPI module in either master or slave mode.  This funtion sets the
 * value of the MSTM bit of the SPICN register based on the input value.
 *
 * Input:     mstm - Use 'MASTER_MODE' to place the SPI module in master mode
 *                   and 'SLAVE_MODE' to place the module in slave mode.
 *
 * Output:    None
 *
 * Example:   To set the MAXQ device as the master...
 *                  spi_setmode(MASTER_MODE);
 *
 ******************************************************************************/
void spi_setmode(const spi_mode mstm);

typedef enum {IDLE_LOW, IDLE_HIGH} spi_polarity;

/******************************************************************************
 * Function:  spi_setclockpolarity
 *
 * Sets the idle logic state of the serial clock and defines which clock edge is
 * the active edge.
 *
 * Input:     ckpol - The new value for the CKPOL bit of the SPICF register.  An
 *                    input of 'IDLE_LOW' will set the rising edge of the serial
 *                    clock to the active edge.  'IDLE_HIGH' sets the falling
 *                    edge as the active one.
 *
 * Output:    None
 *
 * Example:   To set the active clock edge to the rising edge...
 *                  spi_setclockpolarity(IDLE_LOW);
 *
 ******************************************************************************/
void spi_setclockpolarity(const spi_polarity ckpol);

typedef enum {ACTIVE_EDGE, INACTIVE_EDGE} spi_phase;

/******************************************************************************
 * Function:  spi_setclockphase
 *
 * Selects which edge of the clock is used to sample the serial shift data.
 *
 * Input:     ckpha - The new value for the CKPHA bit of the SPICF register.
 *                    When the input is 'ACTIVE_EDGE', data is sampled on the
 *                    active clock edge.  Using an input of 'INACTIVE_EDGE' will
 *                    cause the data to be sampled on the inactive clock edge.
 *
 * Output:    None
 *
 * Example:   To sample data on the active edge of the serial clock...
 *                  spi_setclockphase(ACTIVE_EDGE);
 *
 ******************************************************************************/
void spi_setclockphase(const spi_phase ckpha);

typedef enum {LENGTH_8, LENGTH_16} spi_size;

/******************************************************************************
 * Function:  spi_setcharacterlength
 *
 * Selects the size of the SPI data transfers.
 *
 * Input:     chr - The new value for the CHR bit of the SPICF register.  Use
 *                  'LENGTH_8' for 8-bit transfers and 'LENGTH_16' for 16-bit
 *                  transfers.
 *
 * Output:    None
 *
 * Example:   To enable 16-bit transfers...
 *                  spi_setcharacterlength(LENGTH_16);
 *
 ******************************************************************************/
void spi_setcharacterlength(const spi_size chr);

/******************************************************************************
 * Function:  spi_setbaudrate(int baudrate, int systemfreq)
 *
 * Sets the serial clock baud rate by computing and setting the appropriate
 * value for the SPI Clock Divider Ratio (SPICK).  The buad rate is determined
 * using the following formula:
 *
 *            SPI Baud Rate = System Clock Frequency / (2 * Clock Divider Ratio)
 *
 * Input:     baudrate - the desired baudrate for the SPI serial clock.
 *            systemfreq - the current frequency of the MAXQ's system clock in
 *                         Hz.  This value must account for the system clock
 *                         divide modes, including Power Management Mode.
 *
 * Output:    The actual buad rate achieved.
 *
 ******************************************************************************/
unsigned long spi_setbaudrate(const unsigned long baudrate,
                              const unsigned long systemfreq);

/******************************************************************************
 * Function: spi_enablemodefaultdetection
 *
 * Enables the detection of mode fault errors.  This funtion sets the value of
 * the MODFE bit of the SPICN register based on the input value.
 *
 * Input:     modfe - 0 will disable the mode fault detection and any non-zero
 *                    value will enable it.
 *
 * Output:    None
 *
 * Example:   To enable the mode fault detection...
 *                  spi_enablemodefaultdetection(1);
 *
 ******************************************************************************/
void spi_enablemodefaultdetection(const unsigned char modfe);

#define SPIFLAGS_MODEFAULT       0x08
#define SPIFLAGS_WRITECOLLISION  0x10
#define SPIFLAGS_RECEIVEOVERRUN  0x20
#define SPIFLAGS_TRANFERCOMPLETE 0x40

#define SPIFLAGS_ALLERRORS       0x38
#define SPIFLAGS_ALL             0x78

/******************************************************************************
 * Function:  spi_geterrors
 *
 * Gets the types of SPI system errors that have occurred.  Three types of
 * errors can be detected by the SPI module:
 *      1.  Mode Fault - Occurs when more than one SPI device simultaneously
 *                       tries to be a master.
 *      2.  Write Collision - Occurs when an SPI transfer is attempted while
 *                            another tranfer is already in progress.
 *      3.  Receive Overrun - Occurs when an SPI transfer completes before the
 *                            previous character has been read from the receive
 *                            holding buffer.
 *
 * Input:     clearerrors - if non-zero, the error flags will be cleared after
 *                          they have been read.
 *
 * Output:    A mask indicating which errors have occured.  This mask will be
 *            composed of the SPIFLAGS_MODEFAULT, SPIFLAGS_WRITECOLLISION, and
 *            SPIFLAGS_RECEIVEOVERRUN flags.
 *
 ******************************************************************************/
unsigned char spi_geterrors(const unsigned char clearerrors);

/******************************************************************************
 * Function: spi_enableinterrupts
 *
 * Enables any of the SPI interrupt sources (mode fault, write collition,
 * receive overrun, or transfer complete) to generate interrupt requests.  Note
 * that this function does not enable the interrupt in the IMR mask, only in the
 * SPICF register.  You may still need to enable the module interrupts with the
 * IMR mask.
 *
 * Input:     espii - 0 will disable the interrupt sources and any non-zero
 *                    value will enable them.
 *
 * Output:    None
 *
 * Example:   To enable the interrupts...
 *                  spi_enableinterrupts(1);
 *
 ******************************************************************************/
void spi_enableinterrupts(const unsigned char espii);

/******************************************************************************
 * Function:  spi_getinterruptsources
 *
 * Gets the source of an SPI interrupt.  Four types of interrupts can be occur:
 *      1.  Mode Fault - Occurs when more than one SPI device simultaneously
 *                       tries to be a master.
 *      2.  Write Collision - Occurs when an SPI transfer is attempted while
 *                            another tranfer is already in progress.
 *      3.  Receive Overrun - Occurs when an SPI transfer completes before the
 *                            previous character has been read from the receive
 *                            holding buffer.
 *      4.  Transfer Complete - Occurs at the end of a successful SPI transfer.
 *
 * Input:     clearinterrupts - if non-zero, the interrupt flags will be cleared
 *                              after they have been read.
 *
 * Output:    A mask indicating which interrupts have occured.  This mask will
 *            be composed of the SPIFLAGS_MODEFAULT, SPIFLAGS_WRITECOLLISION,
 *            SPIFLAGS_RECEIVEOVERRUN, and SPIFLAGS_TRANSFERCOMPLETE flags.
 *
 ******************************************************************************/
unsigned char spi_getinterruptsources(const unsigned char clearinterrupts);

/******************************************************************************
 * Function:  spi_transmit
 *
 * Performs a spi transfer.  First, this function waits for the SPI Transfer 
 * Busy flag (STBY) to be clear.  Then, the input data is written to the SPI 
 * Data Buffer (SPIB) register.  This function will then block until the
 * transfer complete flag is asserted.  Finally, the data received is returned
 * and the transfer complete flag is cleared.  Note, if master mode is enabled, 
 * this function initiates the transfer.  When in slave mode, this method could
 * block forever waiting for the master to request a transfer.
 *
 * Input:     ch - the character to transmit
 *
 * Output:    The character received during the transfer.
 *
 ******************************************************************************/
unsigned int spi_transmit(const unsigned int ch);

#endif
